/*
Copyright (C) 2004 Relja Arandjelovic, see COPYING

    This file is part of RA Pai Gaw Poker.

    RA Pai Gaw Poker is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "poker.h"

void raz(Karta &x, Karta &y);
int index(Karta r[], int sta, int poc, int n);

void sortRuku(Karta r[],int n);
int UporediRuke(Karta r1[], Karta r2[]);
int Uporedi2sa2(Karta r1[], Karta r2[]);
int Uporedi2saRukom(Karta r1[], Karta R2[]);

void POKER_funkzaproveru1();
void POKER_funkzaproveru2();

//--------------------------------------------------------

void raz(Karta &x, Karta &y){
  Karta z; z.vred=x.vred; z.boja=x.boja;
  x.vred=y.vred; x.boja=y.boja;
  y.vred=z.vred; y.boja=z.boja;
}

void sortRuku(Karta r[],int n=5){
  int i,j;
  for (i=0;i<n-1;i++)
    for (j=i+1;j<n;j++)
      if (r[i].vred<r[j].vred || 
	  (r[i].vred==r[j].vred && r[i].boja>r[j].boja)) raz(r[i],r[j]);
}

// Trazi sta (=4 za poker, =3 za triling..) koji nije pre poc (zbog 2 para)
int index(Karta r[], int sta, int poc, int n=5){
  int i,j;
  for (i=poc; i<n-sta+1 ; i=j){
    for (j=i+1; j<n && r[j].vred==r[i].vred; j++);
    if (j-i==sta) return i;
  }
  return -1;
}

//----------------------------------------------------------

int PetKeceva(Karta r[], int n=5){
  if (r[0].vred==KEC && r[1].vred==KEC && r[2].vred==KEC && r[3].vred==KEC &&
      r[n-1].vred==0) return 1; return 0;
}

int StraightFlush(int str, int fl){
  if (!fl) return 0;
  return str;
}

int Flush(Karta r[], int n=5){
  int i;
  for (i=1;i<n && (r[i].boja==r[i-1].boja || r[i].boja==0); i++);
  if (i==n) return r[0].vred;
  return 0;
}

int Straight(Karta r[], int n=5){
  int i,rupa=0,dzoker=0;
  if (r[n-1].vred==0) {n--; dzoker=1;}

  for (i=1;i<n;i++){
    if (r[i].vred==r[i-1].vred-1) continue;
    if (r[i].vred==r[i-1].vred-2 && rupa==0) rupa=i; else break;
  }
  if (i<n || (rupa!=0 && dzoker==0)) return 0;
  if (rupa!=0) {
    // ako postoji rupa a i==n znaci da je moguc straight pa
    // stavljam dzokera i dodeljujem mu vrednost
    r[n].vred=r[rupa].vred+1; sortRuku(r);
  } else {
    // ako ne postoji rupa gledam da li postoji dzoker
    if (dzoker){
    // gledam da ga ubacim najbolje - ako najveca karta !A
      if (r[0].vred!=KEC) {
	r[n].vred=r[0].vred+1;
	raz(r[0],r[n]);n++;
	sortRuku(r,n);
      }
      else  // posto jeste onda dzoker zamenjuje najmanju kartu
	r[n].vred=r[n-1].vred-1;
      
    }
  }
  return r[0].vred; // najveca karta u straight-u da bi bio poredak
}

//----------------------------------------------------------

/* Poredak:
5A
Royal Flash (A-10) iste boje
Straight Flush poredjane iste boje
4
3+2
Flush (ista boja)
Straight (redom)
3
2+2
2
1
*/

int UporediRuke(Karta r1[], Karta r2[]){
  int i,j,sta,str1,str2,fl1,fl2;
  int t1=0,t2=0;
  
  if (r1[0].boja==0) r1[0].vred=0; // zbog nekih stvari posle HouseWay
  if (r2[0].boja==0) r2[0].vred=0;

  sortRuku(r1); sortRuku(r2);

  // PROVERA: Pet Keceva
  if (PetKeceva(r1) && !PetKeceva(r2)) {r1[4].vred=KEC; return 1;}
  if (!PetKeceva(r1) && PetKeceva(r2)) {r2[4].vred=KEC; return -1;}
  if (PetKeceva(r1) && PetKeceva(r2)) {r1[4].vred=KEC;r2[4].vred=KEC;return 0;}
  // -

  fl1=Flush(r1);fl2=Flush(r2); // VAZNO je da prvo Flush jer Straight menja poredak
  str1=Straight(r1);str2=Straight(r2);

  
  // PROVERA: Ako ne Straight i postoji Kec zameni sa 1 pa probaj
  if (!str1 && r1[0].vred==KEC) {
    r1[0].vred=1; sortRuku(r1);
    str1=Straight(r1);
    if (!str1) { // nije uspelo, vrati na Kec jer bolje
      for (i=0;i<5 && r1[i].vred!=1;i++);
      r1[i].vred=KEC; sortRuku(r1);
    }
  }
  if (!str2 && r2[0].vred==KEC) {
    r2[0].vred=1; sortRuku(r2);
    str2=Straight(r2);
    if (!str2) { // nije uspelo, vrati na Kec jer bolje
      for (i=0;i<5 && r2[i].vred!=1;i++);
      r2[i].vred=KEC; sortRuku(r2);
    }
  }
  // -
  


  // PROVERA: Royal Flash i Straight Flush
  if (StraightFlush(str1,fl1)>StraightFlush(str2,fl2))
    return 1; // U SF spada i RoyalF
  if (StraightFlush(str1,fl1)<StraightFlush(str2,fl2))
    return -1;
  if (StraightFlush(str1,fl1)>0) return 0; // oba ista - nereseno
  // -

  // Ako nema Straight zameni Dzokera sa Kecom jer to je pravilo
  if (!str1 && r1[4].vred==0) {r1[4].vred=KEC; sortRuku(r1);}
  if (!str2 && r2[4].vred==0) {r2[4].vred=KEC; sortRuku(r2);}
  // -

  for (i=0;i<4;i++)
    for (j=i+1; j<5 && r1[i].vred==r1[j].vred ; j++) t1++;

  for (i=0;i<4;i++)
    for (j=i+1; j<5 && r2[i].vred==r2[j].vred ; j++) t2++;

  /*
t=
6 -poker
4 -full
3 -triling
2 -2 para
1 -par
   */

  // PROVERA: neko flush ili straight
  if ((fl2 || str2) && t1>=4) return 1;
  if ((fl1 || str1) && t2>=4) return -1;
  // -

  // PROVERA: jedan slush drugi straight
  if (fl1 && !fl2 && !str1 && str2) return 1;
  if (fl2 && !fl1 && !str2 && str1) return -1;
  // -

  // PROVERA: neko ima flush ili straight a drugi nema
  if ((fl1 || str1) && !str2 && !fl2) return 1;
  if ((fl2 || str2) && !str1 && !fl1) return -1;
  // -

  // PROVERA: bolji flush
  // automatski ce se odraditi jer posto nema ni parova ni trilinga itd
  // na kraju ce se proveravati pojedinacne karte...
  // -


  // PROVERA: sad vise nema specijalnih slucajeva - proveravaj normalno

  if (t1>t2) return 1;
  if (t1<t2) return -1;

  // ako dobili isto idi redom poker,triling,par - ne treba full jer ce
  // triling pa par da odradi dobro

  
  for(sta=4;sta>0;sta--){
    i=index(r1,sta,0); j=index(r2,sta,0);

    while (i>=0 && r1[i].vred==r2[j].vred){
      i=index(r1,sta,i+1);
      j=index(r2,sta,j+1);
    }
    if (i>=0) break;
  }

  if (sta<0) return 0;
  if (r1[i].vred>r2[j].vred) return 1;
  return -1;
}

int Uporedi2sa2(Karta r1[], Karta r2[]){
  if (r1[0].vred<r1[1].vred) raz(r1[0],r1[1]);
  if (r2[0].vred<r2[1].vred) raz(r2[0],r2[1]);
  if (r1[1].boja==0){ r1[1].vred=KEC; raz(r1[0],r1[1]); }
  if (r2[1].boja==0){ r2[1].vred=KEC; raz(r2[0],r2[1]); }

  if (r1[0].vred==r1[1].vred && r2[0].vred!=r2[1].vred) return 1;
  if (r2[0].vred==r2[1].vred && r1[0].vred!=r1[1].vred) return -1;
  if (r1[0].vred>r2[0].vred) return 1; // radi i ako su oba par jer
  if (r1[0].vred<r2[0].vred) return -1; // taman poredi parove
  if (r1[1].vred>r2[1].vred) return 1;
  if (r1[1].vred<r2[1].vred) return -1;
  return 0;
}

int Uporedi2saRukom(Karta r1[], Karta R2[]){
  int rez;
  
  Karta R1[5];
  R1[0].vred=r1[0].vred; R1[0].boja=r1[0].boja;
  if (R1[0].boja==0) {R1[0].vred=KEC; R1[0].boja=10;}
  R1[1].vred=r1[1].vred; R1[1].boja=r1[1].boja;
  if (R1[1].boja==0) {R1[1].vred=KEC; R1[1].boja=10;}
  R1[2].vred=-3; R1[2].boja=1;
  R1[3].vred=-4; R1[3].boja=2;
  R1[4].vred=-5; R1[4].boja=3;
  sortRuku(R1); sortRuku(R2);

  rez=UporediRuke(R1,R2);
  // stavljao sam boja na 10 da ne bi dzokera koristio kao dzokera
  // nego kao Keca jer samo tako moze posto su 2 karte
  if (R1[0].boja==10) R1[0].boja=0;
  if (R1[1].boja==10) R1[1].boja=0;
  return rez;
}


//----------------------------------------------------------

void POKER_funkzaproveru1(){
  int i,j,n,k;
  Karta ka[100][5];

  ifstream fin("in.txt");
  fin>>n;

  for (i=0;i<n;i++)
    for (j=0;j<5;j++) {
      fin>>ka[i][j].vred;
      if (ka[i][j].vred==0) ka[i][j].boja=0; else
	ka[i][j].boja=j+1;}

  fin.close();

  for (i=0;i<n-1;i++)
    for (j=i+1;j<n;j++) if (UporediRuke(ka[i],ka[j])<0)
     for (k=0;k<5;k++) raz(ka[i][k],ka[j][k]);

  for (i=0;i<n;i++) {
    for (j=0;j<5;j++){
      if (ka[i][j].boja==0) cout<<"*";
      cout<<ka[i][j].vred<<" ";}
    cout<<"\t "<<index(ka[i],4,0)<<" "<<index(ka[i],3,0)<<" "<<index(ka[i],2,0)<<" "<<index(ka[i],1,0);
    cout<<endl;
  }
}

void POKER_funkzaproveru2(){
  int i,j;
  Karta r[2][5];

  while (1){

    cout<<"\nPrvi\n";
    for (i=0;i<5;i++) cin>>r[0][i].vred>>r[0][i].boja;
    cout<<"\nDrugi\n";
    for (i=0;i<5;i++) cin>>r[1][i].vred>>r[1][i].boja;
    cout<<endl;

  if (UporediRuke(r[0],r[1])<0)
     for (i=0;i<5;i++) raz(r[0][i],r[1][i]);

  for (i=0;i<2;i++) {
    for (j=0;j<5;j++){
      if (r[i][j].boja==0) cout<<"*";
      cout<<r[i][j].vred<<"_"<<r[i][j].boja<<" ";}
    cout<<endl;
  }


  }
}
